@@ -56,6 +56,7 @@ struct _MMSimpleConnectPropertiesClass {
 };
 
 GType mm_simple_connect_properties_get_type (void);
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMSimpleConnectProperties, g_object_unref)
 
 MMSimpleConnectProperties *mm_simple_connect_properties_new (void);
 
@@ -75,18 +76,26 @@ void mm_simple_connect_properties_set_ip_type       (MMSimpleConnectProperties *
                                                      MMBearerIpFamily ip_type);
 void mm_simple_connect_properties_set_allow_roaming (MMSimpleConnectProperties *self,
                                                      gboolean allow_roaming);
-void mm_simple_connect_properties_set_number        (MMSimpleConnectProperties *self,
-                                                     const gchar *number);
+void mm_simple_connect_properties_set_rm_protocol   (MMSimpleConnectProperties *self,
+                                                     MMModemCdmaRmProtocol protocol);
 
-const gchar         *mm_simple_connect_properties_get_pin           (MMSimpleConnectProperties *self);
-const gchar         *mm_simple_connect_properties_get_operator_id   (MMSimpleConnectProperties *self);
-const gchar         *mm_simple_connect_properties_get_apn           (MMSimpleConnectProperties *self);
-MMBearerAllowedAuth  mm_simple_connect_properties_get_allowed_auth  (MMSimpleConnectProperties *self);
-const gchar         *mm_simple_connect_properties_get_user          (MMSimpleConnectProperties *self);
-const gchar         *mm_simple_connect_properties_get_password      (MMSimpleConnectProperties *self);
-MMBearerIpFamily     mm_simple_connect_properties_get_ip_type       (MMSimpleConnectProperties *self);
-gboolean             mm_simple_connect_properties_get_allow_roaming (MMSimpleConnectProperties *self);
-const gchar         *mm_simple_connect_properties_get_number        (MMSimpleConnectProperties *self);
+const gchar           *mm_simple_connect_properties_get_pin           (MMSimpleConnectProperties *self);
+const gchar           *mm_simple_connect_properties_get_operator_id   (MMSimpleConnectProperties *self);
+const gchar           *mm_simple_connect_properties_get_apn           (MMSimpleConnectProperties *self);
+MMBearerAllowedAuth    mm_simple_connect_properties_get_allowed_auth  (MMSimpleConnectProperties *self);
+const gchar           *mm_simple_connect_properties_get_user          (MMSimpleConnectProperties *self);
+const gchar           *mm_simple_connect_properties_get_password      (MMSimpleConnectProperties *self);
+MMBearerIpFamily       mm_simple_connect_properties_get_ip_type       (MMSimpleConnectProperties *self);
+gboolean               mm_simple_connect_properties_get_allow_roaming (MMSimpleConnectProperties *self);
+MMModemCdmaRmProtocol  mm_simple_connect_properties_get_rm_protocol   (MMSimpleConnectProperties *self);
+
+#ifndef MM_DISABLE_DEPRECATED
+G_DEPRECATED
+void         mm_simple_connect_properties_set_number (MMSimpleConnectProperties *self,
+                                                      const gchar               *number);
+G_DEPRECATED
+const gchar *mm_simple_connect_properties_get_number (MMSimpleConnectProperties *self);
+#endif
 
 /*****************************************************************************/
 /* ModemManager/libmm-glib/mmcli specific methods */
@@ -100,7 +109,7 @@ MMSimpleConnectProperties *mm_simple_connect_properties_new_from_string (const g
 MMSimpleConnectProperties *mm_simple_connect_properties_new_from_dictionary (GVariant *dictionary,
                                                                              GError **error);
 
-MMBearerProperties *mm_simple_connect_properties_get_bearer_properties (MMSimpleConnectProperties *properties);
+MMBearerProperties *mm_simple_connect_properties_get_bearer_properties (MMSimpleConnectProperties *self);
 
 GVariant *mm_simple_connect_properties_get_dictionary (MMSimpleConnectProperties *self);
 #endif
