@@ -34,18 +34,9 @@
 
 G_DEFINE_TYPE (MMBearerMbim, mm_bearer_mbim, MM_TYPE_BASE_BEARER)
 
-enum {
-    PROP_0,
-    PROP_SESSION_ID,
-    PROP_LAST
-};
-
-static GParamSpec *properties[PROP_LAST];
-
 struct _MMBearerMbimPrivate {
     /* The session ID for this bearer */
     guint32 session_id;
-
     MMPort *data;
 };
 
@@ -1426,8 +1417,7 @@ report_connection_status (MMBaseBearer *self,
 
 MMBaseBearer *
 mm_bearer_mbim_new (MMBroadbandModemMbim *modem,
-                    MMBearerProperties *config,
-                    guint32 session_id)
+                    MMBearerProperties   *config)
 {
     MMBaseBearer *bearer;
 
@@ -1435,9 +1425,8 @@ mm_bearer_mbim_new (MMBroadbandModemMbim *modem,
      * and that means that the object is not async-initable, so we just use
      * g_object_new() here */
     bearer = g_object_new (MM_TYPE_BEARER_MBIM,
-                           MM_BASE_BEARER_MODEM, modem,
+                           MM_BASE_BEARER_MODEM,  modem,
                            MM_BASE_BEARER_CONFIG, config,
-                           MM_BEARER_MBIM_SESSION_ID, (guint)session_id,
                            NULL);
 
     /* Only export valid bearers */
@@ -1446,43 +1435,6 @@ mm_bearer_mbim_new (MMBroadbandModemMbim *modem,
     return bearer;
 }
 
-static void
-set_property (GObject *object,
-              guint prop_id,
-              const GValue *value,
-              GParamSpec *pspec)
-{
-    MMBearerMbim *self = MM_BEARER_MBIM (object);
-
-    switch (prop_id) {
-    case PROP_SESSION_ID:
-        self->priv->session_id = g_value_get_uint (value);
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-        break;
-    }
-}
-
-static void
-get_property (GObject *object,
-              guint prop_id,
-              GValue *value,
-              GParamSpec *pspec)
-{
-    MMBearerMbim *self = MM_BEARER_MBIM (object);
-
-    switch (prop_id) {
-    case PROP_SESSION_ID:
-        g_value_set_uint (value, self->priv->session_id);
-        break;
-    default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-        break;
-    }
-}
-
-
 static void
 mm_bearer_mbim_init (MMBearerMbim *self)
 {
@@ -1512,8 +1464,6 @@ mm_bearer_mbim_class_init (MMBearerMbimClass *klass)
 
     /* Virtual methods */
     object_class->dispose = dispose;
-    object_class->get_property = get_property;
-    object_class->set_property = set_property;
 
     base_bearer_class->connect = _connect;
     base_bearer_class->connect_finish = connect_finish;
@@ -1524,14 +1474,4 @@ mm_bearer_mbim_class_init (MMBearerMbimClass *klass)
     base_bearer_class->reload_stats_finish = reload_stats_finish;
     base_bearer_class->load_connection_status = NULL;
     base_bearer_class->load_connection_status_finish = NULL;
-
-    properties[PROP_SESSION_ID] =
-        g_param_spec_uint (MM_BEARER_MBIM_SESSION_ID,
-                           "Session ID",
-                           "Session ID to use with this bearer",
-                           0,
-                           255,
-                           0,
-                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-    g_object_class_install_property (object_class, PROP_SESSION_ID, properties[PROP_SESSION_ID]);
 }
