@@ -25,18 +25,22 @@
 
 #include "mm-device.h"
 #include "mm-plugin.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 
-G_DEFINE_TYPE (MMDevice, mm_device, G_TYPE_OBJECT);
+static void log_object_iface_init (MMLogObjectInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (MMDevice, mm_device, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
 
 enum {
     PROP_0,
-    PROP_PATH,
-    PROP_UDEV_DEVICE,
+    PROP_UID,
+    PROP_OBJECT_MANAGER,
     PROP_PLUGIN,
     PROP_MODEM,
     PROP_HOTPLUGGED,
     PROP_VIRTUAL,
+    PROP_INHIBITED,
     PROP_LAST
 };
 
@@ -53,11 +57,13 @@ struct _MMDevicePrivate {
     /* Whether the device is real or virtual */
     gboolean virtual;
 
-    /* Device path */
-    gchar *path;
+    /* Unique id */
+    gchar *uid;
 
-    /* Parent UDev device */
-    GUdevDevice *udev_device;
+    /* The object manager */
+    GDBusObjectManagerServer *object_manager;
+
+    /* If USB, device vid/pid */
     guint16 vendor;
     guint16 product;
 
@@ -73,217 +79,112 @@ struct _MMDevicePrivate {
 
     /* The Modem object for this device */
     MMBaseModem *modem;
-
-    /* When exported, a reference to the object manager */
-    GDBusObjectManagerServer *object_manager;
+    gulong       modem_valid_id;
 
     /* Whether the device was hot-plugged. */
     gboolean hotplugged;
 
+    /* Whether the device is inhibited. */
+    gboolean inhibited;
+
     /* Virtual ports */
     gchar **virtual_ports;
+
+    /* Scheduled reprobe */
+    guint reprobe_id;
 };
 
 /*****************************************************************************/
 
 static MMPortProbe *
-device_find_probe_with_device (MMDevice    *self,
-                               GUdevDevice *udev_port,
-                               gboolean lookup_ignored)
+probe_list_lookup_by_device (GList          *port_probes,
+                             MMKernelDevice *kernel_port)
 {
     GList *l;
 
-    for (l = self->priv->port_probes; l; l = g_list_next (l)) {
+    for (l = port_probes; l; l = g_list_next (l)) {
         MMPortProbe *probe = MM_PORT_PROBE (l->data);
 
-        if (   g_udev_device_has_property (udev_port, "DEVPATH_OLD")
-            && g_str_has_suffix (g_udev_device_get_sysfs_path (mm_port_probe_peek_port (probe)),
-                                 g_udev_device_get_property (udev_port, "DEVPATH_OLD")))
-            return probe;
-
-        if (g_str_equal (g_udev_device_get_sysfs_path (mm_port_probe_peek_port (probe)),
-                         g_udev_device_get_sysfs_path (udev_port)))
+        if (mm_kernel_device_cmp (mm_port_probe_peek_port (probe), kernel_port))
             return probe;
     }
+    return NULL;
+}
+
+static MMPortProbe *
+probe_list_lookup_by_name (GList       *port_probes,
+                           const gchar *subsystem,
+                           const gchar *name)
+{
+    GList *l;
+
+    for (l = port_probes; l; l = g_list_next (l)) {
+        MMPortProbe    *probe = MM_PORT_PROBE (l->data);
+        MMKernelDevice *probe_device;
+
+        probe_device = mm_port_probe_peek_port (probe);
+        if ((g_strcmp0 (subsystem, mm_kernel_device_get_subsystem (probe_device)) == 0) &&
+            (g_strcmp0 (name,      mm_kernel_device_get_name      (probe_device)) == 0))
+            return probe;
+    }
+    return NULL;
+}
+
+static MMPortProbe *
+device_find_probe_with_device (MMDevice       *self,
+                               MMKernelDevice *kernel_port,
+                               gboolean        lookup_ignored)
+{
+    MMPortProbe *probe;
+
+    probe = probe_list_lookup_by_device (self->priv->port_probes, kernel_port);
+    if (probe)
+        return probe;
 
     if (!lookup_ignored)
         return NULL;
 
-    for (l = self->priv->ignored_port_probes; l; l = g_list_next (l)) {
-        MMPortProbe *probe = MM_PORT_PROBE (l->data);
-
-        if (   g_udev_device_has_property (udev_port, "DEVPATH_OLD")
-            && g_str_has_suffix (g_udev_device_get_sysfs_path (mm_port_probe_peek_port (probe)),
-                                 g_udev_device_get_property (udev_port, "DEVPATH_OLD")))
-            return probe;
-
-        if (g_str_equal (g_udev_device_get_sysfs_path (mm_port_probe_peek_port (probe)),
-                         g_udev_device_get_sysfs_path (udev_port)))
-            return probe;
-    }
-
-    return NULL;
+    return probe_list_lookup_by_device (self->priv->ignored_port_probes, kernel_port);
 }
 
 gboolean
-mm_device_owns_port (MMDevice    *self,
-                     GUdevDevice *udev_port)
+mm_device_owns_port (MMDevice       *self,
+                     MMKernelDevice *kernel_port)
 {
-    return !!device_find_probe_with_device (self, udev_port, FALSE);
+    return !!device_find_probe_with_device (self, kernel_port, TRUE);
 }
 
-static gboolean
-get_device_ids (GUdevDevice *device,
-                guint16     *vendor,
-                guint16     *product)
+static MMPortProbe *
+device_find_probe_with_name (MMDevice    *self,
+                             const gchar *subsystem,
+                             const gchar *name)
 {
-    GUdevDevice *parent = NULL;
-    const gchar *vid = NULL, *pid = NULL, *parent_subsys;
-    gboolean success = FALSE;
-    char *pci_vid = NULL, *pci_pid = NULL;
+    MMPortProbe *probe;
 
-    parent = g_udev_device_get_parent (device);
-    if (parent) {
-        parent_subsys = g_udev_device_get_subsystem (parent);
-        if (parent_subsys) {
-            if (g_str_equal (parent_subsys, "bluetooth")) {
-                /* Bluetooth devices report the VID/PID of the BT adapter here,
-                 * which isn't really what we want.  Just return null IDs instead.
-                 */
-                success = TRUE;
-                goto out;
-            } else if (g_str_equal (parent_subsys, "pcmcia")) {
-                /* For PCMCIA devices we need to grab the PCMCIA subsystem's
-                 * manfid and cardid, since any IDs on the tty device itself
-                 * may be from PCMCIA controller or something else.
-                 */
-                vid = g_udev_device_get_sysfs_attr (parent, "manf_id");
-                pid = g_udev_device_get_sysfs_attr (parent, "card_id");
-                if (!vid || !pid)
-                    goto out;
-            } else if (g_str_equal (parent_subsys, "platform")) {
-                /* Platform devices don't usually have a VID/PID */
-                success = TRUE;
-                goto out;
-            } else if (g_str_has_prefix (parent_subsys, "usb") &&
-                       (!g_strcmp0 (g_udev_device_get_driver (parent), "qmi_wwan") ||
-                        !g_strcmp0 (g_udev_device_get_driver (parent), "cdc_mbim"))) {
-                /* Need to look for vendor/product in the parent of the QMI/MBIM device */
-                GUdevDevice *qmi_parent;
+    probe = probe_list_lookup_by_name (self->priv->port_probes, subsystem, name);
+    if (probe)
+        return probe;
 
-                qmi_parent = g_udev_device_get_parent (parent);
-                if (qmi_parent) {
-                    vid = g_udev_device_get_property (qmi_parent, "ID_VENDOR_ID");
-                    pid = g_udev_device_get_property (qmi_parent, "ID_MODEL_ID");
-                    g_object_unref (qmi_parent);
-                }
-            } else if (g_str_equal (parent_subsys, "pci")) {
-                const char *pci_id;
-
-                /* We can't always rely on the model + vendor showing up on
-                 * the PCI device's child, so look at the PCI parent.  PCI_ID
-                 * has the format "1931:000C".
-                 */
-                pci_id = g_udev_device_get_property (parent, "PCI_ID");
-                if (pci_id && strlen (pci_id) == 9 && pci_id[4] == ':') {
-                    vid = pci_vid = g_strdup (pci_id);
-                    pci_vid[4] = '\0';
-                    pid = pci_pid = g_strdup (pci_id + 5);
-                }
-            }
-        }
-    }
-
-    if (!vid)
-        vid = g_udev_device_get_property (device, "ID_VENDOR_ID");
-    if (!vid)
-        goto out;
-
-    if (strncmp (vid, "0x", 2) == 0)
-        vid += 2;
-    if (strlen (vid) != 4)
-        goto out;
-
-    if (vendor) {
-        *vendor = (guint16) (mm_utils_hex2byte (vid + 2) & 0xFF);
-        *vendor |= (guint16) ((mm_utils_hex2byte (vid) & 0xFF) << 8);
-    }
-
-    if (!pid)
-        pid = g_udev_device_get_property (device, "ID_MODEL_ID");
-    if (!pid) {
-        *vendor = 0;
-        goto out;
-    }
-
-    if (strncmp (pid, "0x", 2) == 0)
-        pid += 2;
-    if (strlen (pid) != 4) {
-        *vendor = 0;
-        goto out;
-    }
-
-    if (product) {
-        *product = (guint16) (mm_utils_hex2byte (pid + 2) & 0xFF);
-        *product |= (guint16) ((mm_utils_hex2byte (pid) & 0xFF) << 8);
-    }
-
-    success = TRUE;
-
-out:
-    if (parent)
-        g_object_unref (parent);
-    g_free (pci_vid);
-    g_free (pci_pid);
-    return success;
+    return probe_list_lookup_by_name (self->priv->ignored_port_probes, subsystem, name);
 }
 
-const gchar *
-mm_device_utils_get_port_driver (GUdevDevice *udev_port)
+gboolean
+mm_device_owns_port_name (MMDevice    *self,
+                          const gchar *subsystem,
+                          const gchar *name)
 {
-    const gchar *driver, *subsys;
-    const char *name = g_udev_device_get_name (udev_port);
-
-    driver = g_udev_device_get_driver (udev_port);
-    if (!driver) {
-        GUdevDevice *parent;
-
-        parent = g_udev_device_get_parent (udev_port);
-        if (parent)
-            driver = g_udev_device_get_driver (parent);
-
-        /* Check for bluetooth; it's driver is a bunch of levels up so we
-         * just check for the subsystem of the parent being bluetooth.
-         */
-        if (!driver && parent) {
-            subsys = g_udev_device_get_subsystem (parent);
-            if (subsys && !strcmp (subsys, "bluetooth"))
-                driver = "bluetooth";
-        }
-
-        if (parent)
-            g_object_unref (parent);
-    }
-
-    /* Newer kernels don't set up the rfcomm port parent in sysfs,
-     * so we must infer it from the device name.
-     */
-    if (!driver && strncmp (name, "rfcomm", 6) == 0)
-        driver = "bluetooth";
-
-    /* Note: may return NULL! */
-    return driver;
+    return !!device_find_probe_with_name (self, subsystem, name);
 }
 
 static void
-add_port_driver (MMDevice *self,
-                 GUdevDevice *udev_port)
+add_port_driver (MMDevice       *self,
+                 MMKernelDevice *kernel_port)
 {
     const gchar *driver;
     guint n_items;
     guint i;
 
-    driver = mm_device_utils_get_port_driver (udev_port);
+    driver = mm_kernel_device_get_driver (kernel_port);
     if (!driver)
         return;
 
@@ -301,71 +202,71 @@ add_port_driver (MMDevice *self,
     if (!driver)
         return;
 
-    self->priv->drivers = g_realloc (self->priv->drivers,
-                                     (n_items + 2) * sizeof (gchar *));
+    self->priv->drivers = g_realloc (self->priv->drivers, (n_items + 2) * sizeof (gchar *));
     self->priv->drivers[n_items] = g_strdup (driver);
     self->priv->drivers[n_items + 1] = NULL;
 }
 
 void
-mm_device_grab_port (MMDevice    *self,
-                     GUdevDevice *udev_port)
+mm_device_grab_port (MMDevice       *self,
+                     MMKernelDevice *kernel_port)
 {
     MMPortProbe *probe;
 
-    if (mm_device_owns_port (self, udev_port))
+    if (mm_device_owns_port (self, kernel_port))
         return;
 
     /* Get the vendor/product IDs out of the first one that gives us
      * some valid value (it seems we may get NULL reported for VID in QMI
      * ports, e.g. Huawei E367) */
     if (!self->priv->vendor && !self->priv->product) {
-        if (!get_device_ids (udev_port,
-                             &self->priv->vendor,
-                             &self->priv->product)) {
-            mm_dbg ("(%s) could not get vendor/product ID",
-                    self->priv->path);
-        }
+        self->priv->vendor  = mm_kernel_device_get_physdev_vid (kernel_port);
+        self->priv->product = mm_kernel_device_get_physdev_pid (kernel_port);
     }
 
     /* Add new port driver */
-    add_port_driver (self, udev_port);
+    add_port_driver (self, kernel_port);
 
     /* Create and store new port probe */
-    probe = mm_port_probe_new (self, udev_port);
+    probe = mm_port_probe_new (self, kernel_port);
     self->priv->port_probes = g_list_prepend (self->priv->port_probes, probe);
 
     /* Notify about the grabbed port */
-    g_signal_emit (self, signals[SIGNAL_PORT_GRABBED], 0, udev_port);
+    g_signal_emit (self, signals[SIGNAL_PORT_GRABBED], 0, kernel_port);
 }
 
 void
-mm_device_release_port (MMDevice    *self,
-                        GUdevDevice *udev_port)
+mm_device_release_port_name (MMDevice    *self,
+                             const gchar *subsystem,
+                             const gchar *name)
 {
     MMPortProbe *probe;
 
-    probe = device_find_probe_with_device (self, udev_port, TRUE);
+    probe = device_find_probe_with_name (self, subsystem, name);
     if (probe) {
-        /* Found, remove from list and destroy probe */
-        self->priv->port_probes = g_list_remove (self->priv->port_probes, probe);
+        /* Found, remove from lists and destroy probe */
+        if (g_list_find (self->priv->port_probes, probe))
+            self->priv->port_probes = g_list_remove (self->priv->port_probes, probe);
+        else if (g_list_find (self->priv->ignored_port_probes, probe))
+            self->priv->ignored_port_probes = g_list_remove (self->priv->ignored_port_probes, probe);
+        else
+            g_assert_not_reached ();
         g_signal_emit (self, signals[SIGNAL_PORT_RELEASED], 0, mm_port_probe_peek_port (probe));
         g_object_unref (probe);
     }
 }
 
 void
-mm_device_ignore_port  (MMDevice *self,
-                        GUdevDevice *udev_port)
+mm_device_ignore_port  (MMDevice       *self,
+                        MMKernelDevice *kernel_port)
 {
     MMPortProbe *probe;
 
-    probe = device_find_probe_with_device (self, udev_port, FALSE);
+    probe = device_find_probe_with_device (self, kernel_port, FALSE);
     if (probe) {
         /* Found, remove from list and add to the ignored list */
-        mm_dbg ("Fully ignoring port '%s/%s' from now on",
-                g_udev_device_get_subsystem (udev_port),
-                g_udev_device_get_name (udev_port));
+        mm_obj_dbg (self, "fully ignoring port %s from now on",
+                    mm_kernel_device_get_name (kernel_port));
         self->priv->port_probes = g_list_remove (self->priv->port_probes, probe);
         self->priv->ignored_port_probes = g_list_prepend (self->priv->ignored_port_probes, probe);
     }
@@ -387,10 +288,7 @@ unexport_modem (MMDevice *self)
         g_object_set (self->priv->modem,
                       MM_BASE_MODEM_CONNECTION, NULL,
                       NULL);
-
-        mm_dbg ("Unexported modem '%s' from path '%s'",
-                g_udev_device_get_sysfs_path (self->priv->udev_device),
-                path);
+        mm_obj_dbg (self, "unexported modem from path '%s'", path);
         g_free (path);
     }
 }
@@ -401,16 +299,14 @@ static void
 export_modem (MMDevice *self)
 {
     GDBusConnection *connection = NULL;
-    static guint32 id = 0;
-    gchar *path;
+    gchar           *path;
 
     g_assert (MM_IS_BASE_MODEM (self->priv->modem));
     g_assert (G_IS_DBUS_OBJECT_MANAGER (self->priv->object_manager));
 
     /* If modem not yet valid (not fully initialized), don't export it */
     if (!mm_base_modem_get_valid (self->priv->modem)) {
-        mm_dbg ("Modem '%s' not yet fully initialized",
-                g_udev_device_get_sysfs_path (self->priv->udev_device));
+        mm_obj_dbg (self, "modem not yet fully initialized");
         return;
     }
 
@@ -420,14 +316,13 @@ export_modem (MMDevice *self)
                   NULL);
     if (path) {
         g_free (path);
-        mm_dbg ("Modem '%s' already exported",
-                g_udev_device_get_sysfs_path (self->priv->udev_device));
+        mm_obj_dbg (self, "modem already exported");
         return;
     }
 
     /* No outstanding port tasks, so if the modem is valid we can export it */
 
-    path = g_strdup_printf (MM_DBUS_MODEM_PREFIX "/%d", id++);
+    path = g_strdup_printf (MM_DBUS_MODEM_PREFIX "/%d", mm_base_modem_get_dbus_id (self->priv->modem));
     g_object_get (self->priv->object_manager,
                   "connection", &connection,
                   NULL);
@@ -440,27 +335,35 @@ export_modem (MMDevice *self)
     g_dbus_object_manager_server_export (self->priv->object_manager,
                                          G_DBUS_OBJECT_SKELETON (self->priv->modem));
 
-    mm_dbg ("Exported modem '%s' at path '%s'",
-            (self->priv->virtual ?
-             self->priv->path :
-             g_udev_device_get_sysfs_path (self->priv->udev_device)),
-            path);
-
-    /* Once connected, dump additional debug info about the modem */
-    mm_dbg ("(%s): '%s' modem, VID 0x%04X PID 0x%04X (%s)",
-            path,
-            mm_base_modem_get_plugin (self->priv->modem),
-            (mm_base_modem_get_vendor_id (self->priv->modem) & 0xFFFF),
-            (mm_base_modem_get_product_id (self->priv->modem) & 0xFFFF),
-            (self->priv->virtual ?
-             "virtual" :
-             g_udev_device_get_subsystem (self->priv->udev_device)));
+    mm_obj_dbg (self, " exported modem at path '%s'", path);
+    mm_obj_dbg (self, "    plugin:  %s", mm_base_modem_get_plugin (self->priv->modem));
+    mm_obj_dbg (self, "    vid:pid: 0x%04X:0x%04X",
+                (mm_base_modem_get_vendor_id (self->priv->modem) & 0xFFFF),
+                (mm_base_modem_get_product_id (self->priv->modem) & 0xFFFF));
+    if (self->priv->virtual)
+        mm_obj_dbg (self, "    virtual");
 
     g_free (path);
 }
 
 /*****************************************************************************/
 
+static void
+clear_modem (MMDevice *self)
+{
+    if (self->priv->modem_valid_id) {
+        g_signal_handler_disconnect (self->priv->modem, self->priv->modem_valid_id);
+        self->priv->modem_valid_id = 0;
+    }
+
+    if (self->priv->modem) {
+        /* Run dispose before unref-ing, in order to cleanup the SIM object,
+         * if any (which also holds a reference to the modem object) */
+        g_object_run_dispose (G_OBJECT (self->priv->modem));
+        g_clear_object (&(self->priv->modem));
+    }
+}
+
 void
 mm_device_remove_modem (MMDevice  *self)
 {
@@ -468,16 +371,30 @@ mm_device_remove_modem (MMDevice  *self)
         return;
 
     unexport_modem (self);
-
-    /* Run dispose before unref-ing, in order to cleanup the SIM object,
-     * if any (which also holds a reference to the modem object) */
-    g_object_run_dispose (G_OBJECT (self->priv->modem));
-    g_clear_object (&(self->priv->modem));
-    g_clear_object (&(self->priv->object_manager));
+    clear_modem (self);
 }
 
 /*****************************************************************************/
 
+#define REPROBE_SECS 2
+
+static gboolean
+reprobe (MMDevice *self)
+{
+    GError *error = NULL;
+
+    self->priv->reprobe_id = 0;
+
+    mm_obj_dbg (self, "Reprobing modem...");
+    if (!mm_device_create_modem (self, &error)) {
+        mm_obj_warn (self, "could not recreate modem: %s", error->message);
+        g_error_free (error);
+    } else
+        mm_obj_dbg (self, "modem recreated");
+
+    return G_SOURCE_REMOVE;
+}
+
 static void
 modem_valid (MMBaseModem *modem,
              GParamSpec  *pspec,
@@ -486,6 +403,8 @@ modem_valid (MMBaseModem *modem,
     if (!mm_base_modem_get_valid (modem)) {
         /* Modem no longer valid */
         mm_device_remove_modem (self);
+        if (mm_base_modem_get_reprobe (modem))
+            self->priv->reprobe_id = g_timeout_add_seconds (REPROBE_SECS, (GSourceFunc)reprobe, self);
     } else {
         /* Modem now valid, export it, but only if we really have it around.
          * It may happen that the initialization sequence fails because the
@@ -494,17 +413,21 @@ modem_valid (MMBaseModem *modem,
         if (self->priv->modem)
             export_modem (self);
         else
-            mm_dbg ("Not exporting modem; no longer available");
+            mm_obj_dbg (self, "not exporting modem; no longer available");
     }
 }
 
 gboolean
-mm_device_create_modem (MMDevice                  *self,
-                        GDBusObjectManagerServer  *object_manager,
-                        GError                   **error)
+mm_device_create_modem (MMDevice  *self,
+                        GError   **error)
 {
     g_assert (self->priv->modem == NULL);
-    g_assert (self->priv->object_manager == NULL);
+
+    if (self->priv->inhibited) {
+        g_set_error (error, MM_CORE_ERROR, MM_CORE_ERROR_FAILED,
+                     "Device is inhibited");
+        return FALSE;
+    }
 
     if (!self->priv->virtual) {
         if (!self->priv->port_probes) {
@@ -515,9 +438,9 @@ mm_device_create_modem (MMDevice                  *self,
             return FALSE;
         }
 
-        mm_info ("Creating modem with plugin '%s' and '%u' ports",
-                 mm_plugin_get_name (self->priv->plugin),
-                 g_list_length (self->priv->port_probes));
+        mm_obj_info (self, "creating modem with plugin '%s' and '%u' ports",
+                     mm_plugin_get_name (self->priv->plugin),
+                     g_list_length (self->priv->port_probes));
     } else {
         if (!self->priv->virtual_ports) {
             g_set_error (error,
@@ -527,22 +450,18 @@ mm_device_create_modem (MMDevice                  *self,
             return FALSE;
         }
 
-        mm_info ("Creating virtual modem with plugin '%s' and '%u' ports",
-                 mm_plugin_get_name (self->priv->plugin),
-                 g_strv_length (self->priv->virtual_ports));
+        mm_obj_info (self, "creating virtual modem with plugin '%s' and '%u' ports",
+                     mm_plugin_get_name (self->priv->plugin),
+                     g_strv_length (self->priv->virtual_ports));
     }
 
     self->priv->modem = mm_plugin_create_modem (self->priv->plugin, self, error);
-    if (self->priv->modem) {
-        /* Keep the object manager */
-        self->priv->object_manager = g_object_ref (object_manager);
-
+    if (self->priv->modem)
         /* We want to get notified when the modem becomes valid/invalid */
-        g_signal_connect (self->priv->modem,
-                          "notify::" MM_BASE_MODEM_VALID,
-                          G_CALLBACK (modem_valid),
-                          self);
-    }
+        self->priv->modem_valid_id = g_signal_connect (self->priv->modem,
+                                                       "notify::" MM_BASE_MODEM_VALID,
+                                                       G_CALLBACK (modem_valid),
+                                                       self);
 
     return !!self->priv->modem;
 }
@@ -550,9 +469,9 @@ mm_device_create_modem (MMDevice                  *self,
 /*****************************************************************************/
 
 const gchar *
-mm_device_get_path (MMDevice *self)
+mm_device_get_uid (MMDevice *self)
 {
-    return self->priv->path;
+    return self->priv->uid;
 }
 
 const gchar **
@@ -573,20 +492,6 @@ mm_device_get_product (MMDevice *self)
     return self->priv->product;
 }
 
-GUdevDevice *
-mm_device_peek_udev_device (MMDevice *self)
-{
-    g_return_val_if_fail (self->priv->udev_device != NULL, NULL);
-    return self->priv->udev_device;
-}
-
-GUdevDevice *
-mm_device_get_udev_device (MMDevice *self)
-{
-    g_return_val_if_fail (self->priv->udev_device != NULL, NULL);
-    return G_UDEV_DEVICE (g_object_ref (self->priv->udev_device));
-}
-
 void
 mm_device_set_plugin (MMDevice *self,
                       GObject  *plugin)
@@ -629,22 +534,22 @@ mm_device_get_modem (MMDevice *self)
 }
 
 GObject *
-mm_device_peek_port_probe (MMDevice *self,
-                           GUdevDevice *udev_port)
+mm_device_peek_port_probe (MMDevice       *self,
+                           MMKernelDevice *kernel_port)
 {
     MMPortProbe *probe;
 
-    probe = device_find_probe_with_device (self, udev_port, FALSE);
+    probe = device_find_probe_with_device (self, kernel_port, FALSE);
     return (probe ? G_OBJECT (probe) : NULL);
 }
 
 GObject *
-mm_device_get_port_probe (MMDevice *self,
-                          GUdevDevice *udev_port)
+mm_device_get_port_probe (MMDevice       *self,
+                          MMKernelDevice *kernel_port)
 {
     MMPortProbe *probe;
 
-    probe = device_find_probe_with_device (self, udev_port, FALSE);
+    probe = device_find_probe_with_device (self, kernel_port, FALSE);
     return (probe ? g_object_ref (probe) : NULL);
 }
 
@@ -657,11 +562,9 @@ mm_device_peek_port_probe_list (MMDevice *self)
 GList *
 mm_device_get_port_probe_list (MMDevice *self)
 {
-    GList *copy;
-
-    copy = g_list_copy (self->priv->port_probes);
-    g_list_foreach (copy, (GFunc)g_object_ref, NULL);
-    return copy;
+    return g_list_copy_deep (self->priv->port_probes,
+                             (GCopyFunc)g_object_ref,
+                             NULL);
 }
 
 gboolean
@@ -670,6 +573,84 @@ mm_device_get_hotplugged (MMDevice *self)
     return self->priv->hotplugged;
 }
 
+gboolean
+mm_device_get_inhibited (MMDevice *self)
+{
+    return self->priv->inhibited;
+}
+
+/*****************************************************************************/
+
+gboolean
+mm_device_inhibit_finish (MMDevice      *self,
+                          GAsyncResult  *res,
+                          GError       **error)
+{
+    return g_task_propagate_boolean (G_TASK (res), error);
+}
+
+static void
+inhibit_disable_ready (MMBaseModem  *modem,
+                       GAsyncResult *res,
+                       GTask        *task)
+{
+    MMDevice *self;
+    GError   *error = NULL;
+
+    self = g_task_get_source_object (task);
+
+    if (!mm_base_modem_disable_finish (modem, res, &error))
+        g_task_return_error (task, error);
+    else {
+        g_cancellable_cancel (mm_base_modem_peek_cancellable (modem));
+        mm_device_remove_modem (self);
+        g_task_return_boolean (task, TRUE);
+    }
+    g_object_unref (task);
+}
+
+void
+mm_device_inhibit (MMDevice            *self,
+                   GAsyncReadyCallback  callback,
+                   gpointer             user_data)
+{
+    GTask *task;
+
+    task = g_task_new (self, NULL, callback, user_data);
+
+    /* We want to allow inhibiting only devices that are currently
+     * exported in the bus, because otherwise we may be inhibiting
+     * in the middle of port probing and that may lead to some ports
+     * tracked inside the device object during inhibition and some
+     * other ports tracked in the base manager. So, if the device
+     * does not have a valid modem created and exposed, do not allow
+     * the inhibition. */
+    if (!self->priv->modem || !g_dbus_object_get_object_path (G_DBUS_OBJECT (self->priv->modem))) {
+        g_task_return_new_error (task, MM_CORE_ERROR, MM_CORE_ERROR_WRONG_STATE,
+                                 "Modem not exported in the bus");
+        g_object_unref (task);
+        return;
+    }
+
+    /* Flag as inhibited right away */
+    g_assert (!self->priv->inhibited);
+    self->priv->inhibited = TRUE;
+
+    /* Make sure modem is disabled while inhibited */
+    mm_base_modem_disable (self->priv->modem,
+                           (GAsyncReadyCallback)inhibit_disable_ready,
+                           task);
+}
+
+gboolean
+mm_device_uninhibit (MMDevice  *self,
+                     GError   **error)
+{
+    g_assert (self->priv->inhibited);
+    self->priv->inhibited = FALSE;
+    return mm_device_create_modem (self, error);
+}
+
 /*****************************************************************************/
 
 void
@@ -704,29 +685,30 @@ mm_device_is_virtual (MMDevice *self)
 
 /*****************************************************************************/
 
-MMDevice *
-mm_device_new (GUdevDevice *udev_device,
-               gboolean hotplugged)
+static gchar *
+log_object_build_id (MMLogObject *_self)
 {
-    g_return_val_if_fail (udev_device != NULL, NULL);
+    MMDevice *self;
 
-    return MM_DEVICE (g_object_new (MM_TYPE_DEVICE,
-                                    MM_DEVICE_UDEV_DEVICE, udev_device,
-                                    MM_DEVICE_PATH, g_udev_device_get_sysfs_path (udev_device),
-                                    MM_DEVICE_HOTPLUGGED, hotplugged,
-                                    NULL));
+    self = MM_DEVICE (_self);
+    return g_strdup_printf ("device %s", self->priv->uid);
 }
 
+/*****************************************************************************/
+
 MMDevice *
-mm_device_virtual_new (const gchar *path,
-                       gboolean hotplugged)
+mm_device_new (const gchar              *uid,
+               gboolean                  hotplugged,
+               gboolean                  virtual,
+               GDBusObjectManagerServer *object_manager)
 {
-    g_return_val_if_fail (path != NULL, NULL);
+    g_return_val_if_fail (uid != NULL, NULL);
 
     return MM_DEVICE (g_object_new (MM_TYPE_DEVICE,
-                                    MM_DEVICE_PATH, path,
-                                    MM_DEVICE_HOTPLUGGED, hotplugged,
-                                    MM_DEVICE_VIRTUAL, TRUE,
+                                    MM_DEVICE_UID,            uid,
+                                    MM_DEVICE_HOTPLUGGED,     hotplugged,
+                                    MM_DEVICE_VIRTUAL,        virtual,
+                                    MM_DEVICE_OBJECT_MANAGER, object_manager,
                                     NULL));
 }
 
@@ -734,9 +716,7 @@ static void
 mm_device_init (MMDevice *self)
 {
     /* Initialize private data */
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                              MM_TYPE_DEVICE,
-                                              MMDevicePrivate);
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, MM_TYPE_DEVICE, MMDevicePrivate);
 }
 
 static void
@@ -748,20 +728,20 @@ set_property (GObject *object,
     MMDevice *self = MM_DEVICE (object);
 
     switch (prop_id) {
-    case PROP_PATH:
+    case PROP_UID:
         /* construct only */
-        self->priv->path = g_value_dup_string (value);
+        self->priv->uid = g_value_dup_string (value);
         break;
-    case PROP_UDEV_DEVICE:
+    case PROP_OBJECT_MANAGER:
         /* construct only */
-        self->priv->udev_device = g_value_dup_object (value);
+        self->priv->object_manager = g_value_dup_object (value);
         break;
     case PROP_PLUGIN:
         g_clear_object (&(self->priv->plugin));
         self->priv->plugin = g_value_dup_object (value);
         break;
     case PROP_MODEM:
-        g_clear_object (&(self->priv->modem));
+        clear_modem (self);
         self->priv->modem = g_value_dup_object (value);
         break;
     case PROP_HOTPLUGGED:
@@ -770,6 +750,9 @@ set_property (GObject *object,
     case PROP_VIRTUAL:
         self->priv->virtual = g_value_get_boolean (value);
         break;
+    case PROP_INHIBITED:
+        self->priv->inhibited = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -785,8 +768,11 @@ get_property (GObject *object,
     MMDevice *self = MM_DEVICE (object);
 
     switch (prop_id) {
-    case PROP_UDEV_DEVICE:
-        g_value_set_object (value, self->priv->udev_device);
+    case PROP_UID:
+        g_value_set_string (value, self->priv->uid);
+        break;
+    case PROP_OBJECT_MANAGER:
+        g_value_set_object (value, self->priv->object_manager);
         break;
     case PROP_PLUGIN:
         g_value_set_object (value, self->priv->plugin);
@@ -800,6 +786,9 @@ get_property (GObject *object,
     case PROP_VIRTUAL:
         g_value_set_boolean (value, self->priv->virtual);
         break;
+    case PROP_INHIBITED:
+        g_value_set_boolean (value, self->priv->inhibited);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -811,11 +800,18 @@ dispose (GObject *object)
 {
     MMDevice *self = MM_DEVICE (object);
 
-    g_clear_object (&(self->priv->udev_device));
+    if (self->priv->reprobe_id) {
+        g_source_remove (self->priv->reprobe_id);
+        self->priv->reprobe_id = 0;
+    }
+    g_clear_object (&(self->priv->object_manager));
     g_clear_object (&(self->priv->plugin));
-    g_list_free_full (self->priv->port_probes, (GDestroyNotify)g_object_unref);
-    g_list_free_full (self->priv->ignored_port_probes, (GDestroyNotify)g_object_unref);
-    g_clear_object (&(self->priv->modem));
+    g_list_free_full (self->priv->port_probes, g_object_unref);
+    self->priv->port_probes = NULL;
+    g_list_free_full (self->priv->ignored_port_probes, g_object_unref);
+    self->priv->ignored_port_probes = NULL;
+
+    clear_modem (self);
 
     G_OBJECT_CLASS (mm_device_parent_class)->dispose (object);
 }
@@ -825,13 +821,19 @@ finalize (GObject *object)
 {
     MMDevice *self = MM_DEVICE (object);
 
-    g_free (self->priv->path);
+    g_free (self->priv->uid);
     g_strfreev (self->priv->drivers);
     g_strfreev (self->priv->virtual_ports);
 
     G_OBJECT_CLASS (mm_device_parent_class)->finalize (object);
 }
 
+static void
+log_object_iface_init (MMLogObjectInterface *iface)
+{
+    iface->build_id = log_object_build_id;
+}
+
 static void
 mm_device_class_init (MMDeviceClass *klass)
 {
@@ -842,24 +844,24 @@ mm_device_class_init (MMDeviceClass *klass)
     /* Virtual methods */
     object_class->get_property = get_property;
     object_class->set_property = set_property;
-    object_class->finalize = finalize;
-    object_class->dispose = dispose;
+    object_class->finalize     = finalize;
+    object_class->dispose      = dispose;
 
-    properties[PROP_PATH] =
-        g_param_spec_string (MM_DEVICE_PATH,
-                             "Path",
-                             "Device path",
+    properties[PROP_UID] =
+        g_param_spec_string (MM_DEVICE_UID,
+                             "Unique ID",
+                             "Unique device id, e.g. the physical device sysfs path",
                              NULL,
                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-    g_object_class_install_property (object_class, PROP_PATH, properties[PROP_PATH]);
+    g_object_class_install_property (object_class, PROP_UID, properties[PROP_UID]);
 
-    properties[PROP_UDEV_DEVICE] =
-        g_param_spec_object (MM_DEVICE_UDEV_DEVICE,
-                             "UDev Device",
-                             "UDev device object",
-                             G_UDEV_TYPE_DEVICE,
+    properties[PROP_OBJECT_MANAGER] =
+        g_param_spec_object (MM_DEVICE_OBJECT_MANAGER,
+                             "Object manager",
+                             "GDBus object manager server",
+                             G_TYPE_DBUS_OBJECT_MANAGER_SERVER,
                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-    g_object_class_install_property (object_class, PROP_UDEV_DEVICE, properties[PROP_UDEV_DEVICE]);
+    g_object_class_install_property (object_class, PROP_OBJECT_MANAGER, properties[PROP_OBJECT_MANAGER]);
 
     properties[PROP_PLUGIN] =
         g_param_spec_object (MM_DEVICE_PLUGIN,
@@ -893,6 +895,14 @@ mm_device_class_init (MMDeviceClass *klass)
                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property (object_class, PROP_VIRTUAL, properties[PROP_VIRTUAL]);
 
+    properties[PROP_INHIBITED] =
+        g_param_spec_boolean (MM_DEVICE_INHIBITED,
+                              "Inhibited",
+                              "Whether the modem is inhibited",
+                              FALSE,
+                              G_PARAM_READWRITE);
+    g_object_class_install_property (object_class, PROP_INHIBITED, properties[PROP_INHIBITED]);
+
     signals[SIGNAL_PORT_GRABBED] =
         g_signal_new (MM_DEVICE_PORT_GRABBED,
                       G_OBJECT_CLASS_TYPE (object_class),
@@ -900,7 +910,7 @@ mm_device_class_init (MMDeviceClass *klass)
                       G_STRUCT_OFFSET (MMDeviceClass, port_grabbed),
                       NULL, NULL,
                       g_cclosure_marshal_generic,
-                      G_TYPE_NONE, 1, G_UDEV_TYPE_DEVICE);
+                      G_TYPE_NONE, 1, MM_TYPE_KERNEL_DEVICE);
 
     signals[SIGNAL_PORT_RELEASED] =
         g_signal_new (MM_DEVICE_PORT_RELEASED,
@@ -909,5 +919,5 @@ mm_device_class_init (MMDeviceClass *klass)
                       G_STRUCT_OFFSET (MMDeviceClass, port_released),
                       NULL, NULL,
                       g_cclosure_marshal_generic,
-                      G_TYPE_NONE, 1, G_UDEV_TYPE_DEVICE);
+                      G_TYPE_NONE, 1, MM_TYPE_KERNEL_DEVICE);
 }
