@@ -179,12 +179,18 @@ void               mm_shared_qmi_set_primary_sim_slot               (MMIfaceMode
 gboolean           mm_shared_qmi_set_primary_sim_slot_finish        (MMIfaceModem         *self,
                                                                      GAsyncResult         *res,
                                                                      GError              **error);
-void               mm_shared_qmi_setup_sim_hot_swap                 (MMIfaceModem *self,
-                                                                     GAsyncReadyCallback callback,
-                                                                     gpointer user_data);
-gboolean           mm_shared_qmi_setup_sim_hot_swap_finish          (MMIfaceModem *self,
-                                                                     GAsyncResult *res,
-                                                                     GError **error);
+void               mm_shared_qmi_setup_sim_hot_swap                 (MMIfaceModem         *self,
+                                                                     GAsyncReadyCallback   callback,
+                                                                     gpointer              user_data);
+gboolean           mm_shared_qmi_setup_sim_hot_swap_finish          (MMIfaceModem         *self,
+                                                                     GAsyncResult         *res,
+                                                                     GError              **error);
+void               mm_shared_qmi_fcc_unlock                         (MMIfaceModem         *self,
+                                                                     GAsyncReadyCallback   callback,
+                                                                     gpointer              user_data);
+gboolean           mm_shared_qmi_fcc_unlock_finish                  (MMIfaceModem         *self,
+                                                                     GAsyncResult         *res,
+                                                                     GError              **error);
 
 /* Shared QMI location support */
 
